<?php
/**
 * Plugin Name:       Give - Tributes
 * Plugin URI:        https://wordimpress.com
 * Description:       Allow donors to give to your Cause via customizable tributes like "In honor of" or "In memory of".
 * Version:           1.4.2
 * Author:            WordImpress
 * Author URI:        https://wordimpress.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       give-tributes
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Give_Tributes' ) ) :

	/**
	 * Give_Tributes Class
	 *
	 * @package Give_Tributes
	 * @since   1.0
	 */
	final class Give_Tributes {

		/**
		 * Holds the instance
		 *
		 * Ensures that only one instance of Give_Tributes exists in memory at any one
		 * time and it also prevents needing to define globals all over the place.
		 *
		 * TL;DR This is a static property property that holds the singleton instance.
		 *
		 * @var object
		 * @static
		 */
		private static $instance;

		/**
		 * Give Tributes Admin Object.
		 *
		 * @since  1.0
		 * @access public
		 *
		 * @var    Give_Tributes_Admin object.
		 */
		public $plugin_admin;

		/**
		 * Give Tributes Public Object.
		 *
		 * @since  1.0
		 * @access public
		 *
		 * @var    Give_Tributes_Public object.
		 */
		public $plugin_public;

		/**
		 * Get the instance and store the class inside it. This plugin utilises
		 * the PHP singleton design pattern.
		 *
		 * @since     1.0.0
		 * @static
		 * @staticvar array $instance
		 * @access    public
		 *
		 * @see       Give_Tributes();
		 *
		 * @uses      Give_Tributes::hooks() Setup hooks and actions.
		 * @uses      Give_Tributes::includes() Loads all the classes.
		 * @uses      Give_Tributes::licensing() Add Give - Tribute License.
		 *
		 * @return object self::$instance Instance
		 */
		public static function get_instance() {

			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Give_Tributes ) ) {
				self::$instance = new Give_Tributes();
				self::$instance->setup();
			}

			return self::$instance;
		}

		/**
		 * Setup Tribute.
		 *
		 * @since  1.3.4
		 * @access private
		 */
		private function setup() {
			self::$instance->setup_constants();

			add_action( 'give_init', array( $this, 'init' ), 10 );
			add_action( 'plugins_loaded', array( $this, 'check_environment' ), 999 );
		}

		/**
		 * Init Give-Tribute.
		 *
		 * Sets up hooks, licensing and includes files.
		 *
		 * @since  1.0.0
		 * @access public
		 *
		 * @return void
		 */
		public function init() {
			if ( ! self::$instance->check_environment() ) {
				return;
			}

			self::$instance->hooks();
			self::$instance->licensing();
			self::$instance->includes();
		}

		/**
		 * Check plugin environment.
		 *
		 * @since  1.0.0
		 * @access public
		 *
		 * @return bool
		 */
		public function check_environment() {

			// Load plugin helper functions.
			if ( ! function_exists( 'deactivate_plugins' ) || ! function_exists( 'is_plugin_active' ) ) {
				require_once ABSPATH . '/wp-admin/includes/plugin.php';
			}

			// Flag to check whether deactivate plugin or not.
			$is_deactivate = false;

			// Verify dependency cases.
			switch ( true ) {
				case doing_action( 'give_init' ):
					if (
						defined( 'GIVE_VERSION' ) &&
						version_compare( GIVE_VERSION, GIVE_TRIBUTES_MIN_GIVE_VER, '<' )
					) {
						/* Min. Give. plugin version. */
						// Show admin notice.
						$message = sprintf(
							'<strong>%1$s</strong> %2$s <a href="%3$s" target="_blank">%4$s</a> %5$s',
							__( 'Activation Error:', 'give-tributes' ),
							__( 'You must have', 'give-tributes' ),
							esc_url( 'https://givewp.com' ),
							__( 'Give', 'give-tributes' ),
							sprintf( __( 'core version %1$s+ for the Give - Tributes add-on to activate.', 'give-tributes' ), GIVE_TRIBUTES_MIN_GIVE_VER )
						);

						$class = 'notice notice-error';
						printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );

						$is_deactivate = true;
					}

					break;

				case doing_action( 'plugins_loaded' ) && ! did_action( 'give_init' ):
					/* Check to see if Give is activated, if it isn't deactivate and show a banner. */

					// Check for if give plugin activate or not.
					$is_give_active = defined( 'GIVE_PLUGIN_BASENAME' ) ? is_plugin_active( GIVE_PLUGIN_BASENAME ) : false;

					if ( ! $is_give_active ) {
						// Show admin notice.
						$message = sprintf(
							'<strong>%1$s</strong> %2$s <a href="%3$s" target="_blank">%4$s</a> %5$s',
							__( 'Activation Error:', 'give-tributes' ),
							__( 'You must have', 'give-tributes' ),
							esc_url( 'https://givewp.com' ),
							__( 'Give', 'give-tributes' ),
							__( 'plugin installed and activated for Give - Tributes to activate.', 'give-tributes' )
						);

						$class = 'notice notice-error';
						printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );

						$is_deactivate = true;
					}

					break;
			}// End switch().

			// Don't let this plugin activate.
			if ( $is_deactivate ) {
				// Deactivate plugin.
				deactivate_plugins( GIVE_TRIBUTES_BASENAME );

				if ( isset( $_GET['activate'] ) ) {
					unset( $_GET['activate'] );
				}

				return false;
			}

			return true;
		}

		/**
		 * Setup constants.
		 *
		 * @since   1.0.0
		 * @access  private
		 */
		private function setup_constants() {
			if ( ! defined( 'GIVE_TRIBUTES_VERSION' ) ) {
				define( 'GIVE_TRIBUTES_VERSION', '1.4.2' );
			}
			if ( ! defined( 'GIVE_TRIBUTES_MIN_GIVE_VER' ) ) {
				define( 'GIVE_TRIBUTES_MIN_GIVE_VER', '2.1.0' );
			}
			if ( ! defined( 'GIVE_TRIBUTES_SLUG' ) ) {
				define( 'GIVE_TRIBUTES_SLUG', 'give-tributes' );
			}
			if ( ! defined( 'GIVE_TRIBUTES_PLUGIN_FILE' ) ) {
				define( 'GIVE_TRIBUTES_PLUGIN_FILE', __FILE__ );
			}
			if ( ! defined( 'GIVE_TRIBUTES_PLUGIN_DIR' ) ) {
				define( 'GIVE_TRIBUTES_PLUGIN_DIR', dirname( GIVE_TRIBUTES_PLUGIN_FILE ) );
			}
			if ( ! defined( 'GIVE_TRIBUTES_PLUGIN_URL' ) ) {
				define( 'GIVE_TRIBUTES_PLUGIN_URL', plugin_dir_url( GIVE_TRIBUTES_PLUGIN_FILE ) );
			}
			if ( ! defined( 'GIVE_TRIBUTES_BASENAME' ) ) {
				define( 'GIVE_TRIBUTES_BASENAME', plugin_basename( GIVE_TRIBUTES_PLUGIN_FILE ) );
			}
			if ( ! defined( 'GIVE_TRIBUTES_PLUGIN_DIR_PATH' ) ) {
				define( 'GIVE_TRIBUTES_PLUGIN_DIR_PATH', plugin_dir_path( GIVE_TRIBUTES_PLUGIN_FILE ) );
			}
		}

		/**
		 * Throw error on object clone.
		 *
		 * The whole idea of the singleton design pattern is that there is a single
		 * object therefore, we don't want the object to be cloned.
		 *
		 * @since  1.0.0
		 * @access protected
		 *
		 * @return void
		 */
		public function __clone() {
			// Cloning instances of the class is forbidden.
			_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'give-tributes' ), '1.0' );
		}

		/**
		 * Disable Unserialize of the class.
		 *
		 * @since  1.0.0
		 * @access protected
		 *
		 * @return void
		 */
		public function __wakeup() {
			// Unserialize instances of the class is forbidden.
			_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'give-tributes' ), '1.0' );
		}

		/**
		 * Constructor Function.
		 *
		 * @since  1.0.0
		 * @access protected
		 */
		public function __construct() {
			self::$instance = $this;
		}

		/**
		 * Reset the instance of the class
		 *
		 * @since  1.0.0
		 * @access public
		 */
		public static function reset() {
			self::$instance = null;
		}

		/**
		 * Includes.
		 *
		 * @since  1.0.0
		 * @access private
		 */
		private function includes() {

			/**
			 * The class responsible for defining all actions that occur in the admin area.
			 */
			require_once( GIVE_TRIBUTES_PLUGIN_DIR . '/includes/admin/class-give-tributes-admin.php' );

			/**
			 * The class responsible for defining all actions that occur in the public-facing
			 * side of the site.
			 */
			require_once( GIVE_TRIBUTES_PLUGIN_DIR . '/includes/public/class-give-tributes-public.php' );

			/**
			 * Give - Tributes helper functions.
			 */
			require_once( GIVE_TRIBUTES_PLUGIN_DIR . '/includes/give-tributes-helpers.php' );
			require_once( GIVE_TRIBUTES_PLUGIN_DIR . '/includes/give-tributes-ecards.php' );
			require_once( GIVE_TRIBUTES_PLUGIN_DIR . '/includes/give-tributes-mail-cards.php' );

			/**
			 * Give - Register Tributes tags.
			 */
			require_once( GIVE_TRIBUTES_PLUGIN_DIR . '/includes/admin/class-give-tributes-tags.php' );

			self::$instance->plugin_admin  = new Give_Tributes_Admin();
			self::$instance->plugin_public = new Give_Tributes_Public();

		}

		/**
		 * Hooks.
		 *
		 * @since  1.0.0
		 * @access public
		 */
		public function hooks() {
			add_action( 'init', array( $this, 'load_text_domain' ) );
			add_action( 'admin_init', array( $this, 'activation_banner' ) );
			add_filter( 'plugin_action_links_' . GIVE_TRIBUTES_BASENAME, array( $this, 'action_links' ), 10, 2 );
			add_filter( 'plugin_row_meta', array( $this, 'plugin_row_meta' ), 10, 2 );
		}

		/**
		 * Implement Give Licensing for Give - Tribute Add On.
		 *
		 * @since  1.0.0
		 * @access private
		 */
		private function licensing() {
			if ( class_exists( 'Give_License' ) ) {
				new Give_License( GIVE_TRIBUTES_PLUGIN_FILE, 'Tributes', GIVE_TRIBUTES_VERSION, 'WordImpress' );
			}
		}

		/**
		 * Load Plugin Text Domain
		 *
		 * Looks for the plugin translation files in certain directories and loads
		 * them to allow the plugin to be localised
		 *
		 * @since  1.0.0
		 * @access public
		 *
		 * @return bool True on success, false on failure.
		 */
		public function load_text_domain() {
			// Traditional WordPress plugin locale filter.
			$locale = apply_filters( 'plugin_locale', get_locale(), 'give-tributes' );
			$mofile = sprintf( '%1$s-%2$s.mo', 'give-tributes', $locale );

			// Setup paths to current locale file.
			$mofile_local = trailingslashit( GIVE_TRIBUTES_PLUGIN_DIR_PATH . 'languages' ) . $mofile;

			if ( file_exists( $mofile_local ) ) {
				// Look in the /wp-content/plugins/give-tributes/languages/ folder.
				load_textdomain( 'give-tributes', $mofile_local );
			} else {
				// Load the default language files.
				load_plugin_textdomain( 'give-tributes', false, trailingslashit( GIVE_TRIBUTES_PLUGIN_DIR_PATH . 'languages' ) );
			}

			return false;
		}

		/**
		 * Activation banner.
		 *
		 * Uses Give's core activation banners.
		 *
		 * @since 1.0.0
		 *
		 * @return bool
		 */
		public function activation_banner() {

			if ( ! class_exists( 'Give' ) ) {
				return false;
			}

			// Check for activation banner inclusion.
			if ( ! class_exists( 'Give_Addon_Activation_Banner' ) && file_exists( GIVE_PLUGIN_DIR . 'includes/admin/class-addon-activation-banner.php' ) ) {
				include GIVE_PLUGIN_DIR . 'includes/admin/class-addon-activation-banner.php';
			}

			// Initialize activation welcome banner.
			if ( class_exists( 'Give_Addon_Activation_Banner' ) ) {

				// Only runs on admin.
				$args = array(
					'file'              => GIVE_TRIBUTES_PLUGIN_FILE,
					'name'              => __( 'Give-Tributes', 'give-tributes' ),
					'version'           => GIVE_TRIBUTES_VERSION,
					'settings_url'      => admin_url( 'edit.php?post_type=give_forms&page=give-settings&tab=tributes' ),
					'documentation_url' => 'http://docs.givewp.com/addon-tributes/',
					'support_url'       => 'https://givewp.com/support/',
					'testing'           => false,
				);
				new Give_Addon_Activation_Banner( $args );
			}

			return true;

		}

		/**
		 * Adding additional setting page link along plugin's action link.
		 *
		 * @since   1.0.0
		 * @access  public
		 *
		 * @param   array $actions get all actions.
		 *
		 * @return  array       return new action array
		 */
		function action_links( $actions ) {

			if ( ! class_exists( 'Give' ) ) {
				return $actions;
			}

			// Check min Give version.
			if ( defined( 'GIVE_TRIBUTES_MIN_GIVE_VER' ) && version_compare( GIVE_VERSION, GIVE_TRIBUTES_MIN_GIVE_VER, '<' ) ) {
				return $actions;
			}

			$new_actions = array(
				'settings' => sprintf( '<a href="%1$s">%2$s</a>', admin_url( 'edit.php?post_type=give_forms&page=give-settings&tab=tributes' ), __( 'Settings', 'give-tributes' ) ),
			);

			return array_merge( $new_actions, $actions );

		}

		/**
		 * Plugin row meta links.
		 *
		 * @since   1.0.0
		 * @access  public
		 *
		 * @param   array  $plugin_meta An array of the plugin's metadata.
		 * @param   string $plugin_file Path to the plugin file, relative to the plugins directory.
		 *
		 * @return  array  return meta links for plugin.
		 */
		function plugin_row_meta( $plugin_meta, $plugin_file ) {

			// Return if not Give-Tributes plugin.
			if ( $plugin_file !== GIVE_TRIBUTES_BASENAME ) {
				return $plugin_meta;
			}

			$new_meta_links = array(
				sprintf( '<a href="%1$s" target="_blank">%2$s</a>', esc_url( add_query_arg( array(
					'utm_source'   => 'plugins-page',
					'utm_medium'   => 'plugin-row',
					'utm_campaign' => 'admin',
				), 'http://docs.givewp.com/addon-tributes/' ) ), __( 'Documentation', 'give-tributes' ) ),
				sprintf( '<a href="%1$s" target="_blank">%2$s</a>', esc_url( add_query_arg( array(
					'utm_source'   => 'plugins-page',
					'utm_medium'   => 'plugin-row',
					'utm_campaign' => 'admin',
				), 'https://givewp.com/addons/' ) ), __( 'Add-ons', 'give-tributes' ) ),
			);

			return array_merge( $plugin_meta, $new_meta_links );

		}


	} //End Give_Tributes Class.

endif;

/**
 * Loads a single instance of Give-Tribute.
 *
 * This follows the PHP singleton design pattern.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * @example <?php $give_tributes = Give_Tributes(); ?>
 *
 * @since   1.0.0
 *
 * @see     Give_Tributes::get_instance()
 *
 * @return object Give_Tributes Returns an instance of the  class
 */
function give_tributes() {
	return Give_Tributes::get_instance();
}

give_tributes();
